@@ -1,14 +1,15 @@
 from __future__ import unicode_literals
 
 from django.conf import settings
+from django.utils import six
 
-from . import Tags, Warning, register
+from . import Error, Tags, Warning, register
 
 
 @register(Tags.urls)
 def check_url_config(app_configs, **kwargs):
     if getattr(settings, 'ROOT_URLCONF', None):
-        from django.core.urlresolvers import get_resolver
+        from django.urls import get_resolver
         resolver = get_resolver()
         return check_resolver(resolver)
     return []
@@ -18,7 +19,7 @@ def check_resolver(resolver):
     """
     Recursively check the resolver.
     """
-    from django.core.urlresolvers import RegexURLPattern, RegexURLResolver
+    from django.urls import RegexURLPattern, RegexURLResolver
     warnings = []
     for pattern in resolver.url_patterns:
         if isinstance(pattern, RegexURLResolver):
@@ -27,12 +28,41 @@ def check_resolver(resolver):
             warnings.extend(check_resolver(pattern))
         elif isinstance(pattern, RegexURLPattern):
             warnings.extend(check_pattern_name(pattern))
+        else:
+            # This is not a url() instance
+            warnings.extend(get_warning_for_invalid_pattern(pattern))
 
-        warnings.extend(check_pattern_startswith_slash(pattern))
+        if not warnings:
+            warnings.extend(check_pattern_startswith_slash(pattern))
 
     return warnings
 
 
+def get_warning_for_invalid_pattern(pattern):
+    """
+    Return a list containing a warning that the pattern is invalid.
+
+    describe_pattern() cannot be used here, because we cannot rely on the
+    urlpattern having regex or name attributes.
+    """
+    if isinstance(pattern, six.string_types):
+        hint = (
+            "Try removing the string '{}'. The list of urlpatterns should not "
+            "have a prefix string as the first element.".format(pattern)
+        )
+    elif isinstance(pattern, tuple):
+        hint = "Try using url() instead of a tuple."
+    else:
+        hint = None
+
+    return [Error(
+        "Your URL pattern {!r} is invalid. Ensure that urlpatterns is a list "
+        "of url() instances.".format(pattern),
+        hint=hint,
+        id="urls.E004",
+    )]
+
+
 def describe_pattern(pattern):
     """
     Format the URL pattern for display in warning messages.
@@ -67,8 +97,11 @@ def check_pattern_startswith_slash(pattern):
     regex_pattern = pattern.regex.pattern
     if regex_pattern.startswith('/') or regex_pattern.startswith('^/'):
         warning = Warning(
-            "Your URL pattern {} has a regex beginning with a '/'. "
-            "Remove this slash as it is unnecessary.".format(describe_pattern(pattern)),
+            "Your URL pattern {} has a regex beginning with a '/'. Remove this "
+            "slash as it is unnecessary. If this pattern is targeted in an "
+            "include(), ensure the include() pattern has a trailing '/'.".format(
+                describe_pattern(pattern)
+            ),
             id="urls.W002",
         )
         return [warning]
